from mpi4py import MPI 
import numpy as np

def f(x,y):
    return x**3 * y *.3 + .3/4. * x**2 * y**3 - y * x *.4


comm = MPI.COMM_WORLD
size = comm.Get_size() 
rank = comm.Get_rank() 
name = MPI.Get_processor_name()
n = 1e6

assert(n%size == 0)
chunk_size = n/size;

if rank == 0:
    x = np.linspace(1,10,n)
    y = np.linspace(1,20,n)
else:
    x = np.empty(chunk_size, dtype='d')
    y = np.empty(chunk_size, dtype='d')

#scatter x across the processes
comm.Scatter([x, MPI.DOUBLE], [x,MPI.DOUBLE])
comm.Scatter([y, MPI.DOUBLE], [y,MPI.DOUBLE])

if rank == 0:
    result = np.empty(n, dtype='d')
else:
    result = np.empty(chunk_size, dtype='d')

#compute
result[0:chunk_size] = f(x[0:chunk_size],y[0:chunk_size])
comm.Barrier()

#gather result
comm.Gather([result,chunk_size, MPI.DOUBLE], [result,MPI.DOUBLE])

#VERIFY RESULT
if rank == 0:
    x2 = np.linspace(1,10,n)
    y2 = np.linspace(1,20,n)
    result2 = f(x2,y2)
    nrm = np.linalg.norm(result - result2)
    if np.abs(nrm) < 1e-8:
        print "RESULT OKAY"
    else:
        print "WARNING TEST FAILED",nrm
